package PktDump::Analyser::IPv6;

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant ETHERTYPE => 0x86dd;

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my ( $vc ) = unpack( "C", $pkt );
   ( $vc & 0xf0 ) == 0x60 or die "Expected 6\n";

   my ( $vcl, $len, $nh, $hl, $src, $dst ) = unpack(
       "N     n     C    C    a16   a16", $pkt );

   my $class = ( $vcl >> 20 ) & 0xff;
   my $label = $vcl & 0xfffff;

   my $line = sprintf( "IPv6| %s->%s: proto=%d", $self->format_ip6( $src, $opts ), $self->format_ip6( $dst, $opts ), $nh );

   return $line;
}

sub format_ip6
{
   my $self = shift;
   my ( $addr, $opts ) = @_;

   my $str = join ":", map sprintf("%x",$_), unpack "n*", $addr;

   $str =~ s/:(?:0:)+/::/;

   $str;
}

*format_this_protaddr = \&format_ip6;

1;
