package PktDump::Analyser::PPP;

use strict;
use warnings;

use base qw( PktDump::Analyser );

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my $prot;
   if( ( $prot = unpack( "C", $pkt ) ) & 0x01 ) {
      substr $pkt, 0, 1, "";
   }
   else {
      $prot = unpack( "n", $pkt );
      substr $pkt, 0, 2, "";
   }

   my @lines;

   my $analyser;

   if( my $mth = $self->can( sprintf "analyse_%04x", $prot ) ) {
      push @lines, $mth->( $self, $pkt, $opts );
   }
   elsif( eval { $analyser = PktDump::Analyser->find( PPPDLL_PROTO => $prot ) } ) {
      push @lines, $analyser->analyse( $pkt, $opts );
   }
   else {
      push @lines, sprintf "PPP| prot=%04x", $prot;
      push @lines, $self->format_hex( $pkt ) if $opts->{hexdump};
   }

   return $opts->{one_line} ? $lines[0] : @lines;
}

# LCP
sub analyse_c021
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my ( $code, $id, $len, $data ) = unpack( "C C n a*", $pkt );

   if( my $mth = $self->can( sprintf "analyse_lcp_%d", $code ) ) {
      return "LCP| " . $mth->( $self, $code, $id, $data );
   }
   else {
      return sprintf "LCP| code=%02x id=%02x data={%v02x}", $code, $id, $data;
   }
}

sub analyse_lcp_1
{
   return sprintf "Configure-Request %v02x", $_[3];
}

sub analyse_lcp_2
{
   return sprintf "Configure-Ack %v02x", $_[3];
}

sub analyse_lcp_3
{
   return sprintf "Configure-Nak %v02x", $_[3];
}

sub analyse_lcp_4
{
   return sprintf "Configure-Reject %v02x", $_[3];
}

sub analyse_lcp_5
{
   return sprintf "Terminate-Request %v02x", $_[3];
}

sub analyse_lcp_6
{
   return sprintf "Terminate-Ack %v02x", $_[3];
}

sub analyse_lcp_7
{
   return sprintf "Code-Reject %v02x", $_[3];
}

sub analyse_lcp_8
{
   return sprintf "Protocol-Reject %v02x", $_[3];
}

sub analyse_lcp_9
{
   return sprintf "Echo-Request magic=0x%08x", unpack("N", $_[3]);
}

sub analyse_lcp_10
{
   return sprintf "Echo-Reply magic=0x%08x", unpack("N", $_[3]);
}

sub analyse_lcp_11
{
   return sprintf "Discard-Request magic=0x%08x", unpack("N", $_[3]);
}

1;
