package PktDump::Capfile;

use strict;
use warnings;

use Module::Pluggable
   search_path => [ "PktDump::Capfile" ],
   require     => 1;

# Subclass search key
sub handles_format { my $self = shift; $self->FORMAT eq shift() }

sub new
{
   my $class = shift;
   my ( $dir, $path ) = @_;

   if( $class eq __PACKAGE__ ) {
      my $format;

      $format or 
         $path =~ s/^(\w+):// and $format = $1;

      $format or
         $path =~ m/\.(\w+)$/ and $format = $1;

      $format ||= "pcap";

      undef $class;
      $_->handles_format( $format ) and $class = $_, last for plugins;

      defined $class or die "Cannot find a PktDump::Capfile implementation for '$format'\n";
   }

   my $fh;
   if( $dir eq "write" ) {
      open $fh, ">", $path or die "Cannot write $path - $!\n";
   }
   else {
      die "Unknown capfile direction $dir\n";
   }

   $fh->autoflush( 1 );

   my $self = bless {
      fh => $fh,
   }, $class;

   $self->write_header;

   return $self;
}

sub pack
{
   my $self = shift;
   my ( $fmt, @args ) = @_;

   $self->{fh}->print( pack( $fmt, @args ) );
}

1;
