package PktDump::Capfile::Pcap;

use strict;
use warnings;

use base qw( PktDump::Capfile );
use constant FORMAT => "pcap";

sub write_header
{
   my $self = shift;

   $self->pack( "i s s i i i i",
      0xa1b2c3d4, # magic
      2, 4,       # version
      0,          # GMT offset
      6,          # sigfigs
      65535,      # snaplen
      113,        # Linux SLL
   );
}

sub write_packet
{
   my $self = shift;
   my ( $info, $packet ) = @_;

   my $ts_sec  = int $info->{timestamp};
   my $ts_usec = int( ( $info->{timestamp} - $ts_sec ) * 1000000 );

   my $hatype = $info->{hatype};
   # pcap is expecting packets missing their link header here

   my $hdrlen = PktDump::Analyser->for_hatype( $hatype )->hdrlen( $info, $packet );

   my $len = length($packet) - $hdrlen;;

   $len += 16; # Account for SLL header

   $self->pack( "i i i i", $ts_sec, $ts_usec, $len, $len ); # pcap

   $self->pack( "n n n a8 n", $info->{pkttype}, $hatype, length($info->{addr}), $info->{addr}, $info->{proto} ); # Linux SLL

   $self->pack( "a*", substr $packet, $hdrlen );
}

1;
